# Generated by Django 3.1.2 on 2021-02-19 22:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EmailMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message_id', models.CharField(blank=True, default='', max_length=250)),
                ('message_headers', models.TextField()),
                ('content', models.TextField(blank=True, default='')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('file_attachment', models.FileField(blank=True, default='', upload_to='mail_viewer_attachments')),
                ('parent', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='parts', to='mail_viewer_database_backend.emailmessage')),
            ],
            options={
                'db_table': 'mail_viewer_emailmessage',
                'ordering': ('id',),
            },
        ),
        migrations.AddIndex(
            model_name='emailmessage',
            index=models.Index(fields=['message_id'], name='mail_viewer_message_f7eea6_idx'),
        ),
    ]
